@@ -21,7 +21,8 @@
 #include "_libdna.h"
 
 PyDoc_STRVAR(module_doc,
-	     "C-level utilities to work with DNA (particularly bit-packing)");
+	     "C-level utilities to work with DNA, in particular if considering"
+	     "bit-packing.");
 
 
 //#define LIBDNA_DEBUG
@@ -124,6 +125,12 @@ static inline void bytes_to_bit2(const char *read, const Py_ssize_t n,
     }
 }
 
+/*
+ * Convert an array bit2_ptr of bit-packed DNA of length bit2_n
+ * to the characters corresponding to the DNA into the buffer
+ * "dna" (of length "n", specified to be able to spit an error
+ * message out if inconsistency in the lengths).
+ */
 static inline void bit2_to_bytes(const char *bit2_ptr, const Py_ssize_t bit2_n,
 				 char *dna, const Py_ssize_t n)
 {
@@ -350,6 +357,108 @@ static PyObject* bit2revcompl_frombit2bytearray(PyObject *self,
 }
 
 
+/* Convenience function (could be a macro) */
+static inline void make_sliceboundaries(const Py_ssize_t ilow, 
+					const Py_ssize_t ihigh, 
+					SliceBoundaries *slbo)
+{
+  slbo->low_n_padbytes = ((ilow % NGSP_WORD_SIZE) == 0) ? 0 : NGSP_WORD_SIZE - (ilow % NGSP_WORD_SIZE);
+  slbo->low_offset = (slbo->low_n_padbytes == 0)?0:1;
+  slbo->high_n_padbytes = ihigh % NGSP_WORD_SIZE;
+  slbo->high_offset = (slbo->high_n_padbytes == 0)?0:1;
+  slbo->ilow_packed = ilow / NGSP_WORD_SIZE; /* lowest packed index */
+  slbo->ihigh_packed = (ihigh - 1) / NGSP_WORD_SIZE + 1;// - (high_n_padbytes == 0 ? 1 : 0); /* highest packed index */
+}
+
+
+
+static inline void packeddna_fromslice(const Py_ssize_t ilow, 
+				       const Py_ssize_t ihigh,
+				       Packed2BitDNA *packeddna)
+{
+  packeddna->len = ihigh - ilow;
+  packeddna->len_packed = ((ihigh - 1) / NGSP_WORD_SIZE + 1) - ilow / NGSP_WORD_SIZE;
+  packeddna->offset = ((ilow % NGSP_WORD_SIZE) == 0) ? 0 : NGSP_WORD_SIZE - (ilow % NGSP_WORD_SIZE);
+}
+
+
+PyDoc_STRVAR(slicebit2_frombit2bytes_doc,
+"Takes a bytes object with 2-bit packed DNA values, a slice in the DNA space,"
+" and returns a new bytes object with 2-bit packed DNA values."
+" As with other Python slices the last element is not included.");
+
+static PyObject* slicebit2_frombit2bytes(PyObject *self, PyObject *args)
+{
+
+  Py_ssize_t ilow, ihigh;
+  PyObject *packed_bytes;
+  if (!PyArg_ParseTuple(args, "O!nn", &PyBytes_Type, &packed_bytes, 
+			&ilow, &ihigh)) { 
+    return NULL; 
+  }
+  unsigned char *packed_char;
+
+  if (ilow < 0) {
+    PyErr_Format(PyExc_IndexError, 
+		 "Lowest limit for the slice must be positive");
+    return NULL;
+  }
+  if (ihigh < ilow) {
+    PyErr_Format(PyExc_IndexError, 
+		 "Lowest index must be lower that the highest.");
+    return NULL;
+  }
+
+  Py_ssize_t n = PyBytes_GET_SIZE(packed_bytes);
+  /* Offset if boundaries of the slice not multiple of NGSP_WORD_SIZE */
+  SliceBoundaries slbo;
+  make_sliceboundaries(ilow, ihigh, &slbo);
+
+  if (slbo.ihigh_packed > n) {
+    PyErr_Format(PyExc_IndexError, 
+		 "Highest limit for the interval out of bound "
+		 "(requesting %zd while length is %zd)", slbo.ihigh_packed, n);
+    return NULL;
+  }
+
+  /*
+   * |----|-- --|- ---| ----|
+   *    ^----|----|----|-^
+   */
+
+  Py_ssize_t n_slice = (ihigh - ilow) / NGSP_WORD_SIZE;
+  if (((ihigh - ilow) % NGSP_WORD_SIZE) > 0)
+    n_slice++;
+  char *string = NULL;
+  PyObject *slice_bytes = PyBytes_FromStringAndSize(string, n_slice);
+  unsigned char *slice_char = PyBytes_AS_STRING(slice_bytes);
+  packed_char = PyBytes_AS_STRING(packed_bytes);
+  Py_ssize_t i;
+  for (i = 0; i < n_slice; i++) {
+    slice_char[i] = (char)0; 
+  }
+  unsigned short remainder_src, remainder_dest, shift_src, shift_dest;
+  Py_ssize_t i_src, i_dest;
+  unsigned char src;
+  /* printf("%i\n", packed_char[i / NGSP_WORD_SIZE]); */
+  for (i = ilow; i < ihigh; i++) {
+    /* printf("%i\n", i); */
+    remainder_src = i % NGSP_WORD_SIZE;
+    remainder_dest = (i-ilow) % NGSP_WORD_SIZE;
+    i_src = i / NGSP_WORD_SIZE;
+    i_dest = (i-ilow) / NGSP_WORD_SIZE;
+    shift_src = (NGSP_WORD_SIZE - remainder_src - 1) * 2;
+    shift_dest = (NGSP_WORD_SIZE - remainder_dest - 1) * 2;
+    src = (unsigned char)((packed_char[i_src] & (0x03 << shift_src)) >> shift_src);
+    slice_char[i_dest] |= (src << shift_dest);
+    /* printf("src: i=%i - reste=%i - shift=%i - value=%i\n", */
+    /* 	   i_src, remainder_src, shift_src, src); */
+    /* printf("dest: i=%i - reste=%i - shift=%i - new=%i - value=%i\n", */
+    /* 	   i_dest, remainder_dest, shift_dest, (src << shift_dest), slice_char[i_dest]); */
+  }
+  return slice_bytes;
+}
+
 
 
 PyDoc_STRVAR(slice_frombit2bytes_doc,
@@ -359,7 +468,6 @@ static PyObject* slice_frombit2bytes(PyObject *self, PyObject *args)
 
   Py_ssize_t ilow, ihigh;
   PyObject *packed_bytes;
-  const int WORD_SIZE = 4;
   if (!PyArg_ParseTuple(args, "O!nn", &PyBytes_Type, &packed_bytes, 
 			&ilow, &ihigh)) { 
     return NULL; 
@@ -380,21 +488,19 @@ static PyObject* slice_frombit2bytes(PyObject *self, PyObject *args)
   /* if (ilow == Py_None) */
   /*   ilow = 0; */
   /* if (ihigh == Py_None) */
-  /*   ihigh = n * WORD_SIZE; */
+  /*   ihigh = n * NGSP_WORD_SIZE; */
+
+  /* Offset if boundaries of the slice not multiple of NGSP_WORD_SIZE */
+  SliceBoundaries slbo;
+
+  make_sliceboundaries(ilow, ihigh, &slbo);
 
-  /* Offset if boundaries of the slice not multiple of WORD_SIZE */
-  Py_ssize_t low_n_padbytes = ((ilow % WORD_SIZE) == 0) ? 0 : WORD_SIZE - (ilow % WORD_SIZE);
-  int low_offset = (low_n_padbytes == 0)?0:1;
-  Py_ssize_t high_n_padbytes = ihigh % WORD_SIZE;
-  int high_offset = (high_n_padbytes == 0)?0:1;
-  Py_ssize_t ilow_packed = ilow / WORD_SIZE; /* lowest packed index */
-  Py_ssize_t ihigh_packed = (ihigh - 1) / WORD_SIZE + 1;// - (high_n_padbytes == 0 ? 1 : 0); /* highest packed index */
   int k;
 
-  if (ihigh_packed > n) {
+  if (slbo.ihigh_packed > n) {
     PyErr_Format(PyExc_IndexError, 
 		 "Highest limit for the interval out of bound "
-		 "(requesting %zd while length is %zd)", ihigh_packed, n);
+		 "(requesting %zd while length is %zd)", slbo.ihigh_packed, n);
     return NULL;
   }
 
@@ -413,20 +519,25 @@ static PyObject* slice_frombit2bytes(PyObject *self, PyObject *args)
 
   char *packed = PyBytes_AS_STRING(packed_bytes);
 #ifdef LIBDNA_DEBUG
-  printf("-->(%zd, %zd) low: offest %i, pad: %zd, ilow_packed:%zd\n", ilow, ihigh, low_offset, low_n_padbytes, ilow_packed, ihigh_packed);
-  printf("-->(%zd, %zd) high: offest %i, pad: %zd, ihigh_packed:%zd\n", ilow, ihigh, high_offset, high_n_padbytes, ihigh_packed);
+  printf("-->(%zd, %zd) low: offest %i, pad: %zd, ilow_packed:%zd\n", 
+	 ilow, ihigh, slbo.low_offset, slbo.low_n_padbytes, 
+	 slbo.ilow_packed, slbo.ihigh_packed);
+  printf("-->(%zd, %zd) high: offest %i, pad: %zd, ihigh_packed:%zd\n", 
+	 ilow, ihigh, slbo.high_offset, slbo.high_n_padbytes, 
+	 slbo.ihigh_packed);
 #endif
-  if ( ilow_packed+low_offset < ihigh_packed ) {
+  if ( slbo.ilow_packed + slbo.low_offset < slbo.ihigh_packed ) {
 #if defined(LIBDNA_DEBUG)
-    printf(" packed[%zd], length = %zd\n", ilow_packed+low_offset, 
-	   (ihigh_packed - ilow_packed -low_offset-high_offset));
+    printf(" packed[%zd], length = %zd\n", 
+	   slbo.ilow_packed + slbo.low_offset, 
+	   (slbo.ihigh_packed - slbo.ilow_packed - slbo.low_offset - slbo.high_offset));
 #endif
-    bit2_to_bytes(&(packed[ilow_packed+low_offset]),
-		  (ihigh_packed - ilow_packed - low_offset - high_offset),
-		  &(unpacked[low_n_padbytes]), 
+    bit2_to_bytes(&(packed[slbo.ilow_packed + slbo.low_offset]),
+		  (slbo.ihigh_packed - slbo.ilow_packed - slbo.low_offset - slbo.high_offset),
+		  &(unpacked[slbo.low_n_padbytes]), 
 		  ihigh - ilow);
   }
-  char padbytes[WORD_SIZE];
+  char padbytes[NGSP_WORD_SIZE];
 #if defined(LIBDNA_DEBUG)
   printf("-");
   for (k = 0; k < n_up; k++) {
@@ -436,18 +547,18 @@ static PyObject* slice_frombit2bytes(PyObject *self, PyObject *args)
 #endif
 
   /* if offset on lower end, now take care of sub word-sized segment */
-  if (low_offset) {
+  if (slbo.low_offset) {
     //int foo = ((ihigh_packed - ilow_packed) > 1)?1:0;
     int foo = 0;
-    bit2_to_bytes(&(packed[ilow_packed - foo]), 1,
-		  (char *)(&padbytes), WORD_SIZE);
-    for (k = 0; k < low_n_padbytes; k++) {
+    bit2_to_bytes(&(packed[slbo.ilow_packed - foo]), 1,
+		  (char *)(&padbytes), NGSP_WORD_SIZE);
+    for (k = 0; k < slbo.low_n_padbytes; k++) {
 #ifdef LIBDNA_DEBUG
       printf("  --> %i : padbytes[%i] = %c\n", k, 
-	     WORD_SIZE - low_n_padbytes + k, 
-	     padbytes[WORD_SIZE - low_n_padbytes + k]);//;k + WORD_SIZE - low_n_padbytes);
+	     NGSP_WORD_SIZE - slbo.low_n_padbytes + k, 
+	     padbytes[NGSP_WORD_SIZE - slbo.low_n_padbytes + k]);//;k + NGSP_WORD_SIZE - low_n_padbytes);
 #endif
-      unpacked[k] = padbytes[WORD_SIZE - low_n_padbytes + k];
+      unpacked[k] = padbytes[NGSP_WORD_SIZE - slbo.low_n_padbytes + k];
     }
 #ifdef LIBDNA_DEBUG
   printf("post-low-fix\n-");
@@ -459,16 +570,16 @@ static PyObject* slice_frombit2bytes(PyObject *self, PyObject *args)
   }
 
   /* if offset on higher end, now take care of sub word-sized segment */
-  if (high_offset) {
-    bit2_to_bytes(&(packed[ihigh_packed-1]), 1, // 10/3
-		  (char *)(&padbytes), WORD_SIZE);
-    for (k = 0; k < high_n_padbytes; k++) {
+  if (slbo.high_offset) {
+    bit2_to_bytes(&(packed[slbo.ihigh_packed-1]), 1, // 10/3
+		  (char *)(&padbytes), NGSP_WORD_SIZE);
+    for (k = 0; k < slbo.high_n_padbytes; k++) {
 #ifdef LIBDNA_DEBUG
       printf("  --> %i : padbytes[%i] = %c -> %zd\n", k, 
 	     k, 
-	     padbytes[k], ihigh - ilow - high_n_padbytes + k);//;k + WORD_SIZE - low_n_padbytes);
+	     padbytes[k], ihigh - ilow - slbo.high_n_padbytes + k);//;k + NGSP_WORD_SIZE - low_n_padbytes);
 #endif
-      unpacked[ihigh - ilow - high_n_padbytes + k] = padbytes[k];
+      unpacked[ihigh - ilow - slbo.high_n_padbytes + k] = padbytes[k];
     }
 
 #ifdef LIBDNA_DEBUG
@@ -491,7 +602,7 @@ static PyObject* slice_frombit2bytearray(PyObject *self, PyObject *args)
 
   Py_ssize_t ilow, ihigh;
   PyObject *packed_bytes;
-  const int WORD_SIZE = 4;
+  const int NGSP_WORD_SIZE = 4;
   if (!PyArg_ParseTuple(args, "O!nn", &PyByteArray_Type, &packed_bytes, 
 			&ilow, &ihigh)) { 
     return NULL; 
@@ -510,13 +621,13 @@ static PyObject* slice_frombit2bytearray(PyObject *self, PyObject *args)
 
   Py_ssize_t n = PyBytes_GET_SIZE(packed_bytes);
 
-  /* Offset if boundaries of the slice not multiple of WORD_SIZE */
-  Py_ssize_t low_n_padbytes = ((ilow % WORD_SIZE) == 0) ? 0 : WORD_SIZE - (ilow % WORD_SIZE);
+  /* Offset if boundaries of the slice not multiple of NGSP_WORD_SIZE */
+  Py_ssize_t low_n_padbytes = ((ilow % NGSP_WORD_SIZE) == 0) ? 0 : NGSP_WORD_SIZE - (ilow % NGSP_WORD_SIZE);
   int low_offset = (low_n_padbytes == 0)?0:1;
-  Py_ssize_t high_n_padbytes = ihigh % WORD_SIZE;
+  Py_ssize_t high_n_padbytes = ihigh % NGSP_WORD_SIZE;
   int high_offset = (high_n_padbytes == 0)?0:1;
-  Py_ssize_t ilow_packed = ilow / WORD_SIZE; /* lowest packed index */
-  Py_ssize_t ihigh_packed = (ihigh - 1) / WORD_SIZE + 1;// - (high_n_padbytes == 0 ? 1 : 0); /* highest packed index */
+  Py_ssize_t ilow_packed = ilow / NGSP_WORD_SIZE; /* lowest packed index */
+  Py_ssize_t ihigh_packed = (ihigh - 1) / NGSP_WORD_SIZE + 1;// - (high_n_padbytes == 0 ? 1 : 0); /* highest packed index */
   int k;
 
   if (ihigh_packed >= n) {
@@ -538,22 +649,22 @@ static PyObject* slice_frombit2bytearray(PyObject *self, PyObject *args)
 		  &(unpacked[low_n_padbytes]), 
 		  ihigh - ilow);
   }
-  char padbytes[WORD_SIZE];
+  char padbytes[NGSP_WORD_SIZE];
 
   /* if offset on lower end, now take care of sub word-sized segment */
   if (low_offset) {
     int foo = 0;
     bit2_to_bytes(&(packed[ilow_packed - foo]), 1,
-		  (char *)(&padbytes), WORD_SIZE);
+		  (char *)(&padbytes), NGSP_WORD_SIZE);
     for (k = 0; k < low_n_padbytes; k++) {
-      unpacked[k] = padbytes[WORD_SIZE - low_n_padbytes + k];
+      unpacked[k] = padbytes[NGSP_WORD_SIZE - low_n_padbytes + k];
     }
   }
 
   /* if offset on higher end, now take care of sub word-sized segment */
   if (high_offset) {
     bit2_to_bytes(&(packed[ihigh_packed-1]), 1, // 10/3
-		  (char *)(&padbytes), WORD_SIZE);
+		  (char *)(&padbytes), NGSP_WORD_SIZE);
     for (k = 0; k < high_n_padbytes; k++) {
       unpacked[ihigh - ilow - high_n_padbytes + k] = padbytes[k];
     }
@@ -579,6 +690,8 @@ static PyMethodDef libxsq_methods[] = {
    slice_frombit2bytes_doc},
   {"slice_frombit2bytearray", (PyCFunction)slice_frombit2bytearray,  METH_VARARGS,
    slice_frombit2bytearray_doc},
+  {"slicebit2_frombit2bytes", (PyCFunction)slicebit2_frombit2bytes,  METH_VARARGS,
+   slicebit2_frombit2bytes_doc},
   {"bit2complement_frombit2bytes", (PyCFunction)bit2complement_frombit2bytes,  METH_O,
    bit2complement_frombit2bytes_doc},
   {"bit2complement_frombit2bytearray", (PyCFunction)bit2complement_frombit2bytearray,  METH_O,
