@@ -73,6 +73,37 @@ class BitPackingTestCase(unittest.TestCase):
         self.assertRaises(IndexError, _libdna.slice_frombit2bytes,
                          packed, 4, 3)
 
+    def test_slicebit2_frombit2bytes(self):
+        string = b'ATACGCGGCT'+b'GATCGTAGCG'
+        packed = _libdna.bit2_frombytes(string)
+        chunk = _libdna.slicebit2_frombit2bytes(packed, 0, 4)
+        self.assertEqual(string[0:4], _libdna.byte_frombit2bytes(chunk))
+        chunk = _libdna.slicebit2_frombit2bytes(packed, 0, 8)
+        self.assertEqual(string[0:8], _libdna.byte_frombit2bytes(chunk))
+        chunk = _libdna.slicebit2_frombit2bytes(packed, 3, 6)
+        self.assertEqual(string[3:6] + 'A'*(4-(6-3)%4), _libdna.byte_frombit2bytes(chunk))
+        chunk = _libdna.slicebit2_frombit2bytes(packed, 2, 4)
+        self.assertEqual(string[2:4] + 'A'*(4-(4-2)%4), _libdna.byte_frombit2bytes(chunk))
+        chunk = _libdna.slicebit2_frombit2bytes(packed, 0, 11)
+        self.assertEqual(string[0:11] + 'A'*(4-(11-0)%4), 
+                         _libdna.byte_frombit2bytes(chunk))
+        chunk = _libdna.slicebit2_frombit2bytes(packed, 3, 11)
+        self.assertEqual(string[3:11], 
+                         _libdna.byte_frombit2bytes(chunk))
+        chunk = _libdna.slicebit2_frombit2bytes(packed, 4, 11)
+        self.assertEqual(string[4:11] + 'A'*(4-(11-4)%4), 
+                         _libdna.byte_frombit2bytes(chunk))
+        chunk = _libdna.slicebit2_frombit2bytes(packed, 3, 12)
+        self.assertEqual(string[3:12] + 'A'*(4-(12-3)%4),
+                         _libdna.byte_frombit2bytes(chunk))
+
+        #self.assertRaises(IndexError, _libdna.slice_frombit2bytes,
+        #                 packed, -1, 3)
+        self.assertRaises(IndexError, _libdna.slicebit2_frombit2bytes,
+                         packed, 1, 300)
+        self.assertRaises(IndexError, _libdna.slicebit2_frombit2bytes,
+                         packed, 4, 3)
+
     def test_slice_frombit2bytearray(self):
         string = bytearray(b'ATACGCGGCT'+b'GATCGTAGCG')
         packed = _libdna.bit2_frombytearray(string)
@@ -134,8 +165,8 @@ class BitPackingTestCase(unittest.TestCase):
 
     def testBit2reversecomplement_frombit2bytes(self):
         string =    b'ATACGCGGCT'+b'GATCGTAGCG'
-        string_rc = b'TATGCGCCGA'+b'CTAGCATCGC'
-        string_rc = b''.join(string_rc[x] for x in range(len(string)-1, -1, -1))
+        string_c = b'TATGCGCCGA'+b'CTAGCATCGC'
+        string_rc = b''.join(string_c[x] for x in range(len(string)-1, -1, -1))
         vector = _libdna.bit2_frombytes(string)
         vector_rc = _libdna.bit2reversecomplement_frombit2bytes(vector)
         self.assertEqual(len(vector), len(vector_rc))
@@ -144,8 +175,8 @@ class BitPackingTestCase(unittest.TestCase):
 
     def testBit2reversecomplement_frombit2bytearray(self):
         string =    b'ATACGCGGCT'+b'GATCGTAGCG'
-        string_rc = b'TATGCGCCGA'+b'CTAGCATCGC'
-        string_rc = b''.join(string_rc[x] for x in range(len(string)-1, -1, -1))
+        string_c = b'TATGCGCCGA'+b'CTAGCATCGC'
+        string_rc = b''.join(string_c[x] for x in range(len(string)-1, -1, -1))
         vector = _libdna.bit2_frombytearray(bytearray(string))
         vector_rc = _libdna.bit2reversecomplement_frombit2bytearray(vector)
         self.assertEqual(len(vector), len(vector_rc))
