@@ -116,6 +116,21 @@ def make_htmlreport(fqfile, directory = '.', verbose = True,
     if verbose:
         sys.stdout.write('done.\n')
 
+def write_fastq(buf, entry, seq_proc = None, qual_proc = None):
+    if entry.header[0] != '@':
+        buf.write('@')
+    buf.write(entry.header)
+    buf.write('\n')
+    if seq_proc is None:
+        buf.write(entry.sequence)
+    else:
+        buf.write(seq_proc(entry.sequence))
+    buf.write('\n+\n')
+    if qual_proc is None:
+        buf.write(entry.quality)
+    else:
+        buf.write(qual_proc(entry.quality))
+    buf.write('\n')
 
 if __name__ == '__main__':
     import sys, argparse
