@@ -440,15 +440,18 @@ def make_csfasta(lib, fragment, buf_cs, buf_qual, f, flowcell,
     read_count = long(0)
     inc = long(1)
 
-    for tile_i, (tile_n, a) in enumerate(iter_valuearrays(lib, fragment, what='ColorCallQV')):
+    for tile_i, (tile_n, a) in enumerate(iter_valuearrays(lib, fragment, 
+                                                          what='ColorCallQV')):
         if numbase is None:
             numbase = a.shape[1]
         seq_id = seqid_template_csfasta(flowcell, fragment)
         xyloc = lib[tile_n]['Fragments'][u'yxLocation'][:]
         read_count += xyloc.shape[0]
         prefix = FRAGMENT_START[fragment]
-        for row_i in _progress_bar_iter(xrange(a.shape[0]), suffix = ' tile: %i'%(tile_i+1)): 
+        for row_i in _progress_bar_iter(xrange(a.shape[0]), 
+                                        suffix = ' tile: %i'%(tile_i+1)): 
             seq, qual = colourqual_frombuffer(a[row_i])
+            # turn integers to their ASCII representations
             seq = bytearray_addint(seq, 48)
             
             x,y = xyloc[row_i]
@@ -477,7 +480,7 @@ def make_csfastq(lib, fragment, buf, f, flowcell,
                                                           what='ColorCallQV')):
         if numbase is None:
             numbase = a.shape[1]
-        seq_id = seqid_template_csfasta(flowcell, fragment)
+        seq_id = seqid_template(f, flowcell, tile_n)
         xyloc = lib[tile_n]['Fragments'][u'yxLocation'][:]
         read_count += xyloc.shape[0]
         prefix = FRAGMENT_START[fragment]
@@ -489,8 +492,8 @@ def make_csfastq(lib, fragment, buf, f, flowcell,
             x,y = xyloc[row_i]
             buf.write(seq_id %(x,y))
             buf.write('\n')
-            buf_cs.write(prefix)
-            buf_cs.write(seq)
+            buf.write(prefix)
+            buf.write(seq)
             buf.write('\n+\n')
             buf.write(' '.join( str(q) for q in qual))
             buf.write('\n')
