@@ -42,18 +42,18 @@ def main():
                         dest = 'dest_dir',
                         default = '.',
                         help='directory in which resulting files should be put')
-    parser.add_argument('-C', '--CSFASTA',
-                        dest = 'to_csfasta',
-                        action = 'store_true',
-                        help = "output to CSFASTA + QUAL.")
-    parser.add_argument('-Q', '-FASTQ',
-                        dest = 'to_fastq',
-                        action = 'store_true',
-                        help = "output to FASTQ. If -E/--ECC, it will use the ECC base calls, otherwise it will do a naive conversion from the starting base.")
+    parser.add_argument('-f', '--format-style',
+                        dest = 'output_format',
+                        choices = ('FASTA', 'FASTQ'),
+                        help = "Style of the format to output to.")
+    parser.add_argument('-s', '--data-space',
+                        dest = 'data_space',
+                        choices = ('base', 'color'),
+                        help = "Space to output the data into.")
     parser.add_argument('-E', '--ECC',
                         dest = 'use_ecc',
                         action = 'store_true',
-                        help = 'use ECC base calling whenever present (and conversion to FASTQ is asked).')
+                        help = 'use ECC to get data in base space.')
     parser.add_argument('-i', '--info',
                         dest = 'want_info',
                         action = 'store_true',
@@ -64,6 +64,12 @@ def main():
     options = parser.parse_args()
 
 
+    if options.use_ecc and options.data_space == 'color':
+        sys.stderr.write('Error: wanting data in color space is not compatible with wanting ECC data.')
+        sys.stderr.flush()
+        sys.exit(1)
+
+
     for f in options.paths:
         if not os.path.exists(f):
             print('Error: Path "%s" does not exist' %f)
@@ -102,31 +108,39 @@ def main():
                 sys.stdout.flush()
             xf.close()
 
-        if options.to_csfasta:
-            xsq.automagic_csfasta(f, path_out = dest_dir,
-                                  fragment = None, 
-                                  buf_size = long(2E6), 
-                                  out = sys.stdout)
-        if options.to_fastq:
-            if options.use_ecc:
-                try:
-                    xsq.automagic_fastq(f, path_out = dest_dir, 
-                                        fragment = None, 
-                                        buf_size = long(2E6), 
-                                        out = sys.stdout,
-                                        cs = False)
-                except KeyError, ke:
-                    print('\nError: No base calls for file "%s".' %f)
+        if options.output_format == 'FASTA':
+            if options.data_space == 'color':
+                xsq.automagic_csfasta(f, path_out = dest_dir,
+                                      fragment = None, 
+                                      buf_size = long(2E6), 
+                                      out = sys.stdout)
             else:
-                    xsq.automagic_fastq(f, path_out = dest_dir, 
-                                        fragment = None, 
-                                        buf_size = long(2E6), 
-                                        out = sys.stdout,
-                                        extension = 'csfq',
-                                        cs = True)
+                print('--->')   
+        else:
+            # FASTQ data
+            if options.data_space == 'color':
+                xsq.automagic_fastq(f, path_out = dest_dir, 
+                                    fragment = None, 
+                                    buf_size = long(2E6), 
+                                    out = sys.stdout,
+                                    extension = 'csfq',
+                                    cs = True)
+            else:
+                # base space
+                if options.use_ecc:
+                    try:
+                        xsq.automagic_fastq(f, path_out = dest_dir, 
+                                            fragment = None, 
+                                            buf_size = long(2E6), 
+                                            out = sys.stdout,
+                                            cs = False)
+                    except KeyError, ke:
+                        print('\nError: No base calls for file "%s".' %f)
+                else:
+                    print('--->')
 
-    if (not options.to_csfasta) and (not options.to_fastq):
-        print('Error: Neither CSFASTA+QUAL nor FASTQ was wanted, so nothing was done. Goodbye.')
+    if (not options.data_space) or (not options.output_format):
+        print('No output format or data space specified, so nothing was done. Goodbye.')
 
 def htmlreport():
     import argparse
@@ -167,7 +181,7 @@ def htmlreport():
             print('Error: destination directory already existing (but not as a directory')
             sys.exit(1)
     else:
-        if option.force:
+        if options.force:
             sys.stdout.write('Creating directory missing...')
             sys.stdout.flush()
             try:
